@@ -24,9 +24,30 @@
 #define COMPAT_H
 
 /* Local definitions for compatibility with old and/or broken distros... */
+#    ifndef AT_NULL
+#        define AT_NULL			0
+#    endif
+#    ifndef AT_PHDR
+#        define AT_PHDR			3
+#    endif
+#    ifndef AT_PHENT
+#        define AT_PHENT		4
+#    endif
+#    ifndef AT_PHNUM
+#        define AT_PHNUM		5
+#    endif
+#    ifndef AT_BASE
+#        define AT_BASE			7
+#    endif
+#    ifndef AT_ENTRY
+#        define AT_ENTRY		9
+#    endif
 #    ifndef AT_RANDOM
 #        define AT_RANDOM		25
 #    endif
+#    ifndef AT_EXECFN
+#        define AT_EXECFN		31
+#    endif
 #    ifndef AT_SYSINFO
 #        define AT_SYSINFO		32
 #    endif
@@ -51,6 +72,24 @@
 #    ifndef WIFCONTINUED
 #        define WIFCONTINUED(status)	((status) == 0xffff)
 #    endif
+#    ifndef PTRACE_GETREGS
+#        define PTRACE_GETREGS		12
+#    endif
+#    ifndef PTRACE_SETREGS
+#        define PTRACE_SETREGS		13
+#    endif
+#    ifndef PTRACE_GETFPREGS
+#        define PTRACE_GETFPREGS	14
+#    endif
+#    ifndef PTRACE_SETFPREGS
+#        define PTRACE_SETFPREGS	15
+#    endif
+#    ifndef PTRACE_GETFPXREGS
+#        define PTRACE_GETFPXREGS	18
+#    endif
+#    ifndef PTRACE_SETFPXREGS
+#        define PTRACE_SETFPXREGS	19
+#    endif
 #    ifndef PTRACE_SETOPTIONS
 #        define PTRACE_SETOPTIONS	0x4200
 #    endif
@@ -138,6 +177,21 @@
 #    ifndef PTRACE_GETVFPREGS
 #        define PTRACE_GETVFPREGS	27
 #    endif
+#    ifndef PTRACE_ARCH_PRCTL
+#        define PTRACE_ARCH_PRCTL	30
+#    endif
+#    ifndef ARCH_SET_GS
+#        define ARCH_SET_GS	0x1001
+#    endif
+#    ifndef ARCH_SET_FS
+#        define ARCH_SET_FS	0x1002
+#    endif
+#    ifndef ARCH_GET_GS
+#        define ARCH_GET_FS	0x1003
+#    endif
+#    ifndef ARCH_GET_FS
+#        define ARCH_GET_GS	0x1004
+#    endif
 #    ifndef PTRACE_SINGLEBLOCK
 #        define PTRACE_SINGLEBLOCK	33
 #    endif
@@ -150,6 +204,9 @@
 #    ifndef TALLOC_FREE
 #        define TALLOC_FREE(ctx) do { talloc_free(ctx); ctx = NULL; } while(0)
 #    endif
+#    ifndef PR_SET_NAME
+#        define PR_SET_NAME		15
+#    endif
 #    ifndef PR_SET_NO_NEW_PRIVS
 #        define PR_SET_NO_NEW_PRIVS	38
 #    endif
@@ -171,7 +228,20 @@
 #    ifndef F_DUPFD_CLOEXEC
 #        define F_DUPFD_CLOEXEC		1030
 #    endif
+#    ifndef O_RDONLY
+#        define O_RDONLY		00000000
+#    endif
 #    ifndef O_CLOEXEC
 #        define O_CLOEXEC		02000000
 #    endif
+#    ifndef MAP_PRIVATE
+#        define MAP_PRIVATE			0x02
+#    endif
+#    ifndef MAP_FIXED
+#        define MAP_FIXED			0x10
+#    endif
+#    ifndef MAP_ANONYMOUS
+#        define MAP_ANONYMOUS			0x20
+#    endif
+
 #endif /* COMPAT_H */
