@@ -96,6 +96,7 @@ static FilteredSysnum filtered_sysnums[] = {
 	{ PR_lstat,		FILTER_SYSEXIT },
 	{ PR_lstat64,		FILTER_SYSEXIT },
 	{ PR_mknod,		FILTER_SYSEXIT },
+	{ PR_mknodat,		FILTER_SYSEXIT },
 	{ PR_newfstatat,	FILTER_SYSEXIT },
 	{ PR_oldlstat,		FILTER_SYSEXIT },
 	{ PR_oldstat,		FILTER_SYSEXIT },
@@ -118,6 +119,8 @@ static FilteredSysnum filtered_sysnums[] = {
 	{ PR_setuid,		FILTER_SYSEXIT },
 	{ PR_setuid32,		FILTER_SYSEXIT },
 	{ PR_setxattr,		FILTER_SYSEXIT },
+	{ PR_setdomainname,	FILTER_SYSEXIT },
+	{ PR_sethostname,	FILTER_SYSEXIT },
 	{ PR_lsetxattr,		FILTER_SYSEXIT },
 	{ PR_fsetxattr,		FILTER_SYSEXIT },
 	{ PR_stat,		FILTER_SYSEXIT },
@@ -573,9 +576,12 @@ static int handle_sysexit_end(Tracee *tracee, Config *config)
 		POKE_MEM_ID(SYSARG_3, sgid);
 		return 0;
 
+	case PR_setdomainname:
+	case PR_sethostname:
 	case PR_setgroups:
 	case PR_setgroups32:
 	case PR_mknod:
+	case PR_mknodat:
 	case PR_capset:
 	case PR_setxattr:
 	case PR_lsetxattr:
@@ -684,45 +690,6 @@ static int handle_sysexit_end(Tracee *tracee, Config *config)
 		return 0;
 	}
 
-	case PR_execve: {
-		ElfAuxVector *vectors;
-		ElfAuxVector *vector;
-		word_t vectors_address;
-
-		/* Override only if it succeed.  */
-		result = peek_reg(tracee, CURRENT, SYSARG_RESULT);
-		if ((int) result < 0)
-			return 0;
-
-		vectors_address = get_elf_aux_vectors_address(tracee);
-		if (vectors_address == 0)
-			return 0;
-
-		vectors = fetch_elf_aux_vectors(tracee, vectors_address);
-		if (vectors == NULL)
-			return 0;
-
-		vector = find_elf_aux_vector(vectors, AT_UID);
-		if (vector != NULL)
-			vector->value = config->ruid;
-
-		vector = find_elf_aux_vector(vectors, AT_EUID);
-		if (vector != NULL)
-			vector->value = config->euid;
-
-		vector = find_elf_aux_vector(vectors, AT_GID);
-		if (vector != NULL)
-			vector->value = config->rgid;
-
-		vector = find_elf_aux_vector(vectors, AT_EGID);
-		if (vector != NULL)
-			vector->value = config->egid;
-
-		push_elf_aux_vectors(tracee, vectors, vectors_address);
-
-		return 0;
-	}
-
 	default:
 		return 0;
 	}
@@ -737,6 +704,53 @@ static int handle_sysexit_end(Tracee *tracee, Config *config)
 #undef SETRESXID
 #undef SETFSXID
 
+/**
+ * Adjust some ELF auxiliary vectors.  This function assumes the
+ * "argv, envp, auxv" stuff is pointed to by @tracee's stack pointer,
+ * as expected right after a successful call to execve(2).
+ */
+static int adjust_elf_auxv(Tracee *tracee, Config *config)
+{
+	ElfAuxVector *vectors;
+	ElfAuxVector *vector;
+	word_t vectors_address;
+
+	vectors_address = get_elf_aux_vectors_address(tracee);
+	if (vectors_address == 0)
+		return 0;
+
+	vectors = fetch_elf_aux_vectors(tracee, vectors_address);
+	if (vectors == NULL)
+		return 0;
+
+	for (vector = vectors; vector->type != AT_NULL; vector++) {
+		switch (vector->type) {
+		case AT_UID:
+			vector->value = config->ruid;
+			break;
+
+		case AT_EUID:
+			vector->value = config->euid;
+			break;
+
+		case AT_GID:
+			vector->value = config->rgid;
+			break;
+
+		case AT_EGID:
+			vector->value = config->egid;
+			break;
+
+		default:
+			break;
+		}
+	}
+
+	push_elf_aux_vectors(tracee, vectors, vectors_address);
+
+	return 0;
+}
+
 /**
  * Handler for this @extension.  It is triggered each time an @event
  * occurred.  See ExtensionEvent for the meaning of @data1 and @data2.
@@ -835,6 +849,19 @@ int fake_id0_callback(Extension *extension, ExtensionEvent event, intptr_t data1
 		return handle_sysexit_end(tracee, config);
 	}
 
+	case SYSCALL_EXIT_START: {
+		Tracee *tracee = TRACEE(extension);
+		Config *config = talloc_get_type_abort(extension->config, Config);
+		word_t result = peek_reg(tracee, CURRENT, SYSARG_RESULT);;
+		word_t sysnum = get_sysnum(tracee, ORIGINAL);
+
+		/* Note: this can be done only before PRoot pushes the
+		 * load script into tracee's stack.  */
+		if ((int) result >= 0 && sysnum == PR_execve)
+			adjust_elf_auxv(tracee, config);
+		return 0;
+	}
+
 	default:
 		return 0;
 	}
