@@ -30,6 +30,7 @@
 #include "syscall/socket.h"
 #include "syscall/chain.h"
 #include "syscall/heap.h"
+#include "execve/execve.h"
 #include "tracee/tracee.h"
 #include "tracee/reg.h"
 #include "tracee/mem.h"
@@ -250,7 +251,7 @@ void translate_syscall_exit(Tracee *tracee)
 		}
 		else {
 			old_reg = SYSARG_2;
-			new_reg = SYSARG_3;
+			new_reg = SYSARG_4;
 		}
 
 		/* Get the old path, then convert it to the same
@@ -425,13 +426,7 @@ void translate_syscall_exit(Tracee *tracee)
 #endif
 
 	case PR_execve:
-		if ((int) syscall_result >= 0) {
-			/* New processes have no heap.  */
-			bzero(tracee->heap, sizeof(Heap));
-	case PR_rt_sigreturn:
-	case PR_sigreturn:
-			tracee->restore_original_regs = false;
-		}
+		translate_execve_exit(tracee);
 		goto end;
 
 	case PR_ptrace:
@@ -440,18 +435,8 @@ void translate_syscall_exit(Tracee *tracee)
 
 	case PR_wait4:
 	case PR_waitpid:
-		if (tracee->as_ptracer.waits_in != WAITS_IN_PROOT) {
-			pid_t pid;
-
-			/* See ptrace/wait.c for explanation.  */
-			pid = (pid_t) syscall_result;
-			if (pid > 0 && is_exited_direct_ptracee(tracee, pid)) {
-				remove_exited_direct_ptracee(tracee, pid);
-				restart_original_syscall(tracee);
-			}
-
+		if (tracee->as_ptracer.waits_in != WAITS_IN_PROOT)
 			goto end;
-		}
 
 		status = translate_wait_exit(tracee);
 		break;
