@@ -37,7 +37,6 @@
 #include <string.h>        /* memcpy(3), */
 #include <stddef.h>        /* offsetof(3), */
 #include <stdint.h>        /* uint*_t, UINT*_MAX, */
-#include <talloc.h>        /* talloc_*, */
 #include <assert.h>        /* assert(3), */
 
 #include "syscall/seccomp.h"
@@ -45,7 +44,7 @@
 #include "syscall/syscall.h"
 #include "syscall/sysnum.h"
 #include "extension/extension.h"
-#include "cli/notice.h"
+#include "cli/note.h"
 
 #include "compat.h"
 #include "attribute.h"
@@ -385,9 +384,7 @@ static FilteredSysnum proot_sysnums[] = {
 	{ PR_rename,		FILTER_SYSEXIT },
 	{ PR_renameat,		FILTER_SYSEXIT },
 	{ PR_rmdir,		0 },
-	{ PR_rt_sigreturn,	FILTER_SYSEXIT },
 	{ PR_setxattr,		0 },
-	{ PR_sigreturn,		FILTER_SYSEXIT },
 	{ PR_socketcall,	FILTER_SYSEXIT },
 	{ PR_stat,		0 },
 	{ PR_stat64,		0 },
