@@ -2,7 +2,7 @@
  *
  * This file is part of PRoot.
  *
- * Copyright (C) 2013 STMicroelectronics
+ * Copyright (C) 2014 STMicroelectronics
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,21 +23,25 @@
 #ifndef SECCOMP_H
 #define SECCOMP_H
 
+#include "syscall/sysnum.h"
 #include "tracee/tracee.h"
 #include "attribute.h"
 #include "arch.h"
 
 typedef struct {
-	word_t value;
-	int flag;
-} FilteredSyscall;
+	Sysnum value;
+	word_t flags;
+} FilteredSysnum;
 
 typedef struct {
-	int architecture;
-	const FilteredSyscall *syscalls;
-} Filter;
+	unsigned int value;
+	size_t nb_abis;
+	Abi abis[NB_MAX_ABIS];
+} SeccompArch;
 
-#define FILTERED_SYSCALL_END { SYSCALL_AVOIDER, -1 }
+#define FILTERED_SYSNUM_END { PR_void, 0 }
+
+#define FILTER_SYSEXIT  0x1
 
 extern int enable_syscall_filtering(const Tracee *tracee);
 
