@@ -35,7 +35,7 @@
 #include <talloc.h>     /* talloc_*, */
 
 #include "tracee/event.h"
-#include "cli/notice.h"
+#include "cli/note.h"
 #include "path/path.h"
 #include "path/binding.h"
 #include "syscall/syscall.h"
@@ -48,18 +48,24 @@
 #include "compat.h"
 
 /**
- * Launch the first process as specified by @tracee->cmdline[].  This
- * function returns -errno if an error occurred, otherwise 0.
+ * Start @tracee->exe with the given @argv[].  This function
+ * returns -errno if an error occurred, otherwise 0.
  */
-int launch_process(Tracee *tracee)
+int launch_process(Tracee *tracee, char *const argv[])
 {
+	char *const default_argv[] = { "-sh", NULL };
 	long status;
 	pid_t pid;
 
+	/* Warn about open file descriptors. They won't be
+	 * translated until they are closed. */
+	if (tracee->verbose > 0)
+		list_open_fd(tracee);
+
 	pid = fork();
 	switch(pid) {
 	case -1:
-		notice(tracee, ERROR, SYSTEM, "fork()");
+		note(tracee, ERROR, SYSTEM, "fork()");
 		return -errno;
 
 	case 0: /* child */
@@ -67,15 +73,10 @@ int launch_process(Tracee *tracee)
 		 * requested program. */
 		status = ptrace(PTRACE_TRACEME, 0, NULL, NULL);
 		if (status < 0) {
-			notice(tracee, ERROR, SYSTEM, "ptrace(TRACEME)");
+			note(tracee, ERROR, SYSTEM, "ptrace(TRACEME)");
 			return -errno;
 		}
 
-		/* Warn about open file descriptors. They won't be
-		 * translated until they are closed. */
-		if (tracee->verbose > 0)
-			list_open_fd(tracee);
-
 		/* Synchronize with the tracer's event loop.  Without
 		 * this trick the tracer only sees the "return" from
 		 * the next execve(2) so PRoot wouldn't handle the
@@ -92,7 +93,7 @@ int launch_process(Tracee *tracee)
 		 * guest rootfs.  Note: Valgrind can't handle execve(2) on
 		 * "foreign" binaries (ENOEXEC) but can handle execvp(3) on such
 		 * binaries.  */
-		execvp(tracee->exe, tracee->cmdline);
+		execvp(tracee->exe, argv[0] != NULL ? argv : default_argv);
 		return -errno;
 
 	default: /* parent */
@@ -109,7 +110,7 @@ int launch_process(Tracee *tracee)
  * signal.  */
 static void kill_all_tracees2(int signum, siginfo_t *siginfo UNUSED, void *ucontext UNUSED)
 {
-	notice(NULL, WARNING, INTERNAL, "signal %d received from process %d",
+	note(NULL, WARNING, INTERNAL, "signal %d received from process %d",
 		signum, siginfo->si_pid);
 	kill_all_tracees();
 
@@ -228,7 +229,7 @@ static void check_architecture(Tracee *tracee)
 	if (!IS_CLASS64(elf_header) || sizeof(word_t) == sizeof(uint64_t))
 		return;
 
-	notice(tracee, ERROR, USER,
+	note(tracee, ERROR, USER,
 		"'%s' is a 64-bit program whereas this version of "
 		"%s handles 32-bit programs only", path, tracee->tool_name);
 
@@ -239,9 +240,9 @@ static void check_architecture(Tracee *tracee)
 	if (strcmp(utsname.machine, "x86_64") != 0)
 		return;
 
-	notice(tracee, INFO, USER,
-		"use a 64-bit version of %s instead, it supports both 32 and 64-bit programs",
-		tracee->tool_name);
+	note(tracee, INFO, USER,
+		"Get a 64-bit version that supports 32-bit binaries here: "
+		"http://static.proot.me/proot-x86_64");
 }
 
 /**
@@ -257,7 +258,7 @@ int event_loop()
 	/* Kill all tracees when exiting.  */
 	status = atexit(kill_all_tracees);
 	if (status != 0)
-		notice(NULL, WARNING, INTERNAL, "atexit() failed");
+		note(NULL, WARNING, INTERNAL, "atexit() failed");
 
 	/* All signals are blocked when the signal handler is called.
 	 * SIGINFO is used to know which process has signaled us and
@@ -266,7 +267,7 @@ int event_loop()
 	signal_action.sa_flags = SA_SIGINFO | SA_RESTART;
 	status = sigfillset(&signal_action.sa_mask);
 	if (status < 0)
-		notice(NULL, WARNING, SYSTEM, "sigfillset()");
+		note(NULL, WARNING, SYSTEM, "sigfillset()");
 
 	/* Handle all signals.  */
 	for (signum = 0; signum < SIGRTMAX; signum++) {
@@ -308,7 +309,7 @@ int event_loop()
 
 		status = sigaction(signum, &signal_action, NULL);
 		if (status < 0 && errno != EINVAL)
-			notice(NULL, WARNING, SYSTEM, "sigaction(%d)", signum);
+			note(NULL, WARNING, SYSTEM, "sigaction(%d)", signum);
 	}
 
 	while (1) {
@@ -317,11 +318,14 @@ int event_loop()
 		int signal;
 		pid_t pid;
 
+		/* This is the only safe place to free tracees.  */
+		free_terminated_tracees();
+
 		/* Wait for the next tracee's stop. */
 		pid = waitpid(-1, &tracee_status, __WALL);
 		if (pid < 0) {
 			if (errno != ECHILD) {
-				notice(NULL, ERROR, SYSTEM, "waitpid()");
+				note(NULL, ERROR, SYSTEM, "waitpid()");
 				return EXIT_FAILURE;
 			}
 			break;
@@ -385,11 +389,13 @@ int handle_tracee_event(Tracee *tracee, int tracee_status)
 	if (WIFEXITED(tracee_status)) {
 		last_exit_status = WEXITSTATUS(tracee_status);
 		VERBOSE(tracee, 1, "pid %d: exited with status %d", pid, last_exit_status);
+		tracee->terminated = true;
 	}
 	else if (WIFSIGNALED(tracee_status)) {
 		check_architecture(tracee);
 		VERBOSE(tracee, (int) (last_exit_status != -1),
 			"pid %d: terminated with signal %d", pid, WTERMSIG(tracee_status));
+		tracee->terminated = true;
 	}
 	else if (WIFSTOPPED(tracee_status)) {
 		/* Don't use WSTOPSIG() to extract the signal
@@ -430,7 +436,7 @@ int handle_tracee_event(Tracee *tracee, int tracee_status)
 				status = ptrace(PTRACE_SETOPTIONS, tracee->pid, NULL,
 						default_ptrace_options);
 				if (status < 0) {
-					notice(tracee, ERROR, SYSTEM, "ptrace(PTRACE_SETOPTIONS)");
+					note(tracee, ERROR, SYSTEM, "ptrace(PTRACE_SETOPTIONS)");
 					exit(EXIT_FAILURE);
 				}
 			}
@@ -586,13 +592,11 @@ bool restart_tracee(Tracee *tracee, int signal)
 	/* Restart the tracee and stop it at the next instruction, or
 	 * at the next entry or exit of a system call. */
 	status = ptrace(tracee->restart_how, tracee->pid, NULL, signal);
-	if (status < 0) {
-		/* The process died in a syscall.  */
-		TALLOC_FREE(tracee);
-		return false;
-	}
+	if (status < 0)
+		return false; /* The process likely died in a syscall.  */
 
 	tracee->restart_how = 0;
 	tracee->running = true;
+
 	return true;
 }
