@@ -34,12 +34,13 @@
 
 #include "tracee/tracee.h"
 #include "tracee/reg.h"
+#include "path/binding.h"
 #include "syscall/sysnum.h"
 #include "tracee/event.h"
 #include "ptrace/ptrace.h"
 #include "ptrace/wait.h"
 #include "extension/extension.h"
-#include "cli/notice.h"
+#include "cli/note.h"
 
 #include "compat.h"
 
@@ -53,15 +54,28 @@ static Tracees tracees;
  */
 static int remove_zombie(Tracee *zombie)
 {
-	Tracee *ptracer;
-
-	ptracer = talloc_get_type_abort(talloc_parent(zombie), Tracee);
-	PTRACER.nb_ptracees--;
-
 	LIST_REMOVE(zombie, link);
 	return 0;
 }
 
+/**
+ * Perform some specific treatments against @pointer according to its
+ * type, before it gets unlinked from @tracee_->life_context.
+ */
+static void clean_life_span_object(const void *pointer, int depth UNUSED,
+				int max_depth UNUSED, int is_ref UNUSED, void *tracee_)
+{
+	Binding *binding;
+	Tracee *tracee;
+
+	tracee = talloc_get_type_abort(tracee_, Tracee);
+
+	/* So far, only bindings need a special treatment.  */
+	binding = talloc_get_type(pointer, Binding);
+	if (binding != NULL)
+		remove_binding_from_all_lists(tracee, binding);
+}
+
 /**
  * Remove @tracee from the list of tracees and update all of its
  * children & ptracees, and its ptracer.  Note: this is a talloc
@@ -71,11 +85,14 @@ static int remove_tracee(Tracee *tracee)
 {
 	Tracee *relative;
 	Tracee *ptracer;
-	int status;
 	int event;
 
 	LIST_REMOVE(tracee, link);
 
+	/* Clean objects that are linked to this tracee's life
+	 * span.  */
+	talloc_report_depth_cb(tracee->life_context, 0, 100, clean_life_span_object, tracee);
+
 	/* This could be optimize by using a dedicated list of
 	 * children and ptracees.  */
 	LIST_FOREACH(relative, &tracees, link) {
@@ -107,10 +124,6 @@ static int remove_tracee(Tracee *tracee)
 	if (ptracer == NULL)
 		return 0;
 
-	/* Sanity checks.  */
-	assert(ptracer != tracee);
-	assert(PTRACER.nb_ptracees > 0);
-
 	/* Zombify this ptracee until its ptracer is notified about
 	 * its death.  */
 	event = tracee->as_ptracee.event4.ptracer.value;
@@ -123,29 +136,31 @@ static int remove_tracee(Tracee *tracee)
 			LIST_INSERT_HEAD(&PTRACER.zombies, zombie, link);
 			talloc_set_destructor(zombie, remove_zombie);
 
+			zombie->parent = tracee->parent;
+			zombie->clone = tracee->clone;
+			zombie->pid = tracee->pid;
+
+			detach_from_ptracer(tracee);
+			attach_to_ptracer(zombie, ptracer);
+
 			zombie->as_ptracee.event4.ptracer.pending = true;
 			zombie->as_ptracee.event4.ptracer.value = event;
 			zombie->as_ptracee.is_zombie = true;
-			zombie->clone = tracee->clone;
-			zombie->pid = tracee->pid;
 
 			return 0;
 		}
 		/* Fallback to the common path.  */
 	}
 
+	detach_from_ptracer(tracee);
+
 	/* Wake its ptracer if there's nothing else to wait for.  */
-	PTRACER.nb_ptracees--;
 	if (PTRACER.nb_ptracees == 0 && PTRACER.wait_pid != 0) {
 		/* Update the return value of ptracer's wait(2).  */
 		poke_reg(ptracer, SYSARG_RESULT, -ECHILD);
 
 		/* Don't forget to write its register cache back.  */
-		status = push_regs(ptracer);
-		if (status < 0) {
-			TALLOC_FREE(ptracer);
-			return 0;
-		}
+		(void) push_regs(ptracer);
 
 		PTRACER.wait_pid = 0;
 		(void) restart_tracee(ptracer, 0);
@@ -207,6 +222,8 @@ static Tracee *new_tracee(pid_t pid)
 	tracee->pid = pid;
 	LIST_INSERT_HEAD(&tracees, tracee, link);
 
+	tracee->life_context = talloc_new(tracee);
+
 	return tracee;
 }
 
@@ -318,6 +335,24 @@ Tracee *get_tracee(const Tracee *current_tracee, pid_t pid, bool create)
 	return (create ? new_tracee(pid) : NULL);
 }
 
+/**
+ * Free all tracees marked as terminated.
+ */
+void free_terminated_tracees()
+{
+	Tracee *next;
+
+	/* Items can't be deleted when using LIST_FOREACH.  */
+	next = tracees.lh_first;
+	while (next != NULL) {
+		Tracee *tracee = next;
+		next = tracee->link.le_next;
+
+		if (tracee->terminated)
+			TALLOC_FREE(tracee);
+	}
+}
+
 /**
  * Make new @parent's child inherit from it.  Depending on
  * @clone_flags, some information are copied or shared.  This function
@@ -347,20 +382,19 @@ int new_child(Tracee *parent, word_t clone_flags)
 	/* Get the pid of the parent's new child.  */
 	status = ptrace(PTRACE_GETEVENTMSG, parent->pid, NULL, &pid);
 	if (status < 0 || pid == 0) {
-		notice(parent, WARNING, SYSTEM, "ptrace(GETEVENTMSG)");
+		note(parent, WARNING, SYSTEM, "ptrace(GETEVENTMSG)");
 		return status;
 	}
 
 	child = get_tracee(parent, (pid_t) pid, true);
 	if (child == NULL) {
-		notice(parent, WARNING, SYSTEM, "running out of memory");
+		note(parent, WARNING, SYSTEM, "running out of memory");
 		return -ENOMEM;
 	}
 
 	/* Sanity checks.  */
 	assert(child != NULL
 	    && child->exe == NULL
-	    && child->cmdline == NULL
 	    && child->fs->cwd == NULL
 	    && child->fs->bindings.pending == NULL
 	    && child->fs->bindings.guest == NULL
@@ -404,12 +438,18 @@ int new_child(Tracee *parent, word_t clone_flags)
 	 *
 	 * -- clone(2) man-page
 	 */
-	child->clone = ((clone_flags & CLONE_PARENT) != 0);
-	if (child->clone)
+	if ((clone_flags & CLONE_PARENT) != 0)
 		child->parent = parent->parent;
 	else
 		child->parent = parent;
 
+	/* Remember if this child belongs to the same thread group as
+	 * its parent.  This is currently useful for ptrace emulation
+	 * only but it deserves to be extended to support execve(2)
+	 * specificity (ie. when a thread calls execve(2), its pid
+	 * gets replaced by the pid of its thread group leader).  */
+	child->clone = ((clone_flags & CLONE_THREAD) != 0);
+
 	/* Depending on how the new process is created, it may be
 	 * automatically traced by the parent's tracer.  */
 	ptrace_options = ( clone_flags == 0			? PTRACE_O_TRACEFORK
@@ -419,9 +459,7 @@ int new_child(Tracee *parent, word_t clone_flags)
 	if (parent->as_ptracee.ptracer != NULL
 	    && (   (ptrace_options & parent->as_ptracee.options) != 0
 		|| (clone_flags & CLONE_PTRACE) != 0)) {
-		Tracee *ptracer = parent->as_ptracee.ptracer;
-		child->as_ptracee.ptracer = ptracer;
-		PTRACER.nb_ptracees++;
+		attach_to_ptracer(child, parent->as_ptracee.ptracer);
 
 		/* All these flags are inheritable, no matter why this
 		 * child is being traced.  */
@@ -474,12 +512,10 @@ int new_child(Tracee *parent, word_t clone_flags)
 		child->fs->bindings.host  = talloc_reference(child->fs, parent->fs->bindings.host);
 	}
 
-	/* The path to the executable and the command-line are unshared only
-	 * once the child process does a call to execve(2).  */
+	/* The path to the executable is unshared only once the child
+	 * process does a call to execve(2).  */
 	child->exe = talloc_reference(child, parent->exe);
-	child->cmdline = talloc_reference(child, parent->cmdline);
 
-	child->qemu_pie_workaround = parent->qemu_pie_workaround;
 	child->qemu = talloc_reference(child, parent->qemu);
 	child->glue = talloc_reference(child, parent->glue);
 
@@ -524,7 +560,6 @@ int new_child(Tracee *parent, word_t clone_flags)
 static void reparent_config(Tracee *new_parent, Tracee *old_parent)
 {
 	new_parent->verbose = old_parent->verbose;
-	new_parent->qemu_pie_workaround = old_parent->qemu_pie_workaround;
 
 #define REPARENT(field) do {							\
 		talloc_reparent(old_parent, new_parent, old_parent->field);	\
@@ -533,7 +568,6 @@ static void reparent_config(Tracee *new_parent, Tracee *old_parent)
 
 	REPARENT(fs);
 	REPARENT(exe);
-	REPARENT(cmdline);
 	REPARENT(qemu);
 	REPARENT(glue);
 	REPARENT(extensions);
