@@ -41,8 +41,8 @@ typedef enum {
 } RegVersion;
 
 struct bindings;
+struct load_info;
 struct extensions;
-struct direct_ptracees;
 struct chained_syscalls;
 
 /* Information related to a file-system name-space.  */
@@ -67,6 +67,7 @@ typedef struct {
 	word_t base;
 	size_t size;
 	size_t prealloc_size;
+	bool disabled;
 } Heap;
 
 /* Information related to a tracee process. */
@@ -84,6 +85,10 @@ typedef struct tracee {
 	/* Is it currently running or not?  */
 	bool running;
 
+	/* Is this tracee ready to be freed?  TODO: move to a list
+	 * dedicated to terminated tracees instead.  */
+	bool terminated;
+
 	/* Parent of this tracee, NULL if none.  */
 	struct tracee *parent;
 
@@ -95,8 +100,6 @@ typedef struct tracee {
 		size_t nb_ptracees;
 		LIST_HEAD(zombies, tracee) zombies;
 
-		struct direct_ptracees *direct_ptracees;
-
 		pid_t wait_pid;
 		word_t wait_options;
 
@@ -119,10 +122,10 @@ typedef struct tracee {
 		} event4;
 
 		bool tracing_started;
-		bool ignore_syscall;
+		bool ignore_loader_syscalls;
+		bool ignore_syscalls;
 		word_t options;
 		bool is_zombie;
-		bool is_load_pending;
 	} as_ptracee;
 
 	/* Current status:
@@ -184,6 +187,11 @@ typedef struct tracee {
 		word_t final_result;
 	} chain;
 
+	/* Load info generated during execve sysenter and used during
+	 * execve sysexit.  */
+	struct load_info *load_info;
+
+
 	/**********************************************************************
 	 * Private but inherited resources                                    *
 	 **********************************************************************/
@@ -208,15 +216,14 @@ typedef struct tracee {
 	/* Virtual heap, emulated with a regular memory mapping.  */
 	Heap *heap;
 
+
 	/**********************************************************************
 	 * Shared resources until the tracee makes a call to execve().        *
 	 **********************************************************************/
 
 	/* Path to the executable, à la /proc/self/exe.  */
 	char *exe;
-
-	/* Initial command-line, à la /proc/self/cmdline.  */
-	char **cmdline;
+	char *new_exe;
 
 
 	/**********************************************************************
@@ -226,9 +233,6 @@ typedef struct tracee {
 	/* Runner command-line.  */
 	char **qemu;
 
-	/* Can the ELF interpreter for QEMU be safely skipped?  */
-	bool qemu_pie_workaround;
-
 	/* Path to glue between the guest rootfs and the host rootfs.  */
 	const char *glue;
 
@@ -262,8 +266,8 @@ extern Tracee *get_stopped_ptracee(const Tracee *ptracer, pid_t pid,
 extern bool has_ptracees(const Tracee *ptracer, pid_t pid, word_t wait_options);
 extern int new_child(Tracee *parent, word_t clone_flags);
 extern Tracee *new_dummy_tracee(TALLOC_CTX *context);
+extern void free_terminated_tracees();
 extern int swap_config(Tracee *tracee1, Tracee *tracee2);
-extern int parse_config(Tracee *tracee, size_t argc, char *argv[]);
 extern void kill_all_tracees();
 
 #endif /* TRACEE_H */
