@@ -2,4 +2,4 @@ if [ ! -x ${ROOTFS}/bin/true ]; then
     exit 125;
 fi
 
-${PROOT} ${ROOTFS} /bin/true
+${PROOT} -r ${ROOTFS} /bin/true
