@@ -7,12 +7,12 @@ if [ $? -eq 0 ]; then
     exit 125;
 fi
 
-export PROOT_NO_SECCOMP=1
-
 ${PROOT} ${PROOT_RAW} -0 id -u                 | grep ^0$
+${PROOT} ${PROOT_RAW} -i 123:456 id -u         | grep ^123$
 ${PROOT} ${PROOT_RAW} -k 3.33.333 uname -r     | grep ^3\.33\.333$
 
 ${PROOT} -0       ${PROOT_RAW} id -u           | grep ^0$
+${PROOT} -i 123:456 ${PROOT_RAW} id -u         | grep ^123$
 ${PROOT} -k 3.33.333 ${PROOT_RAW} uname -r     | grep ^3\.33\.333$
 
 ${PROOT} -0 ${PROOT_RAW} -k 3.33.333 id -u     | grep ^0$
@@ -20,3 +20,6 @@ ${PROOT} -0 ${PROOT_RAW} -k 3.33.333 uname -r  | grep ^3\.33\.333$
 
 ${PROOT} -k 3.33.333 ${PROOT_RAW} -0 id -u     | grep ^0$
 ${PROOT} -k 3.33.333 ${PROOT_RAW} -0 uname -r  | grep ^3\.33\.333$
+
+${PROOT} -i 123:456 ${PROOT_RAW} -k 3.33.333 id -u | grep ^123$
+${PROOT} -k 3.33.333 ${PROOT_RAW} -i 123:456 id -u | grep ^123$
