@@ -2,5 +2,5 @@ if [ ! -x  ${ROOTFS}/bin/true ]; then
     exit 125;
 fi
 
-${PROOT} -w /bin ${ROOTFS} ./true
+${PROOT} -w /bin -r ${ROOTFS} ./true
 
