@@ -2,8 +2,10 @@ if [ -z `which mcookie` ] || [ -z `which rm` ] || [ -z `which ln` ] || [ -z `whi
     exit 125;
 fi
 
-! realpath -e /proc/self/exe
-if [ $? != 1 ]; then
+CHECK1=$(realpath -e /proc/self/exe)
+CHECK2=$(realpath /proc/self/exe)
+
+if [ "${CHECK1}" != "${CHECK2}" ]; then
     exit 125;
 fi
 
